#!/usr/bin/env python
# -*- coding: utf-8 -*-s
import unittest

from sqlalchemy import Column, Integer, String, Text, create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

from . import FullText, FullTextSearch, FullTextForMigration
import modes as FullTextMode

FULLTEXT_TABLE = "test_full_text"
FULLTEXT_TABLE_FOR_MIGRATION = "test_full_text_for_migration"
BASE = declarative_base()

try:
    ENGINE = create_engine('mysql+mysqldb://travis@localhost/test_full_text?charset=utf8', echo=True)
except ImportError:
    ENGINE = create_engine('mysql+pymysql://travis@localhost/test_full_text?charset=utf8', echo=True)

SESSION = sessionmaker(bind=ENGINE)()
SESSION.execute('DROP TABLE IF EXISTS {0};'.format(FULLTEXT_TABLE))
SESSION.execute('DROP TABLE IF EXISTS {0};'.format(FULLTEXT_TABLE_FOR_MIGRATION))

MYSQL_ENGINES = [i[0] for i in SESSION.execute('SHOW ENGINES;').fetchall()
                if i[1] == "YES"]
MYSQL_VERSION = SESSION.execute('SHOW VARIABLES LIKE "version";').fetchone()[1]

class RecipeReviewModel(FullText, BASE):

    __tablename__ = FULLTEXT_TABLE
    # mroonga engine supporting CJK chars
    __table_args__ = {'mysql_engine':'MyISAM',
                      'mysql_charset':'utf8'}

    __fulltext_columns__ = ('commentor','review')

    id        = Column(Integer,primary_key=True)
    commentor = Column(String(length=100))
    review    = Column(Text())

    def __init__(self, commentor, review):
        self.review = review
        self.commentor = commentor


class RecipeReviewModelForMigration(FullTextForMigration, BASE):
    __tablename__ = FULLTEXT_TABLE_FOR_MIGRATION
    # mroonga engine supporting CJK chars
    __table_args__ = {'mysql_engine': 'MyISAM',
                      'mysql_charset': 'utf8'}

    __fulltext_columns__ = ('commentor', 'review')

    id = Column(Integer, primary_key=True)
    commentor = Column(String(length=100))
    review = Column(Text())

    def __init__(self, commentor, review):
        self.review = review
        self.commentor = commentor


RecipeReviewModelForMigration.index_fulltext()


class TestSQLAlchemyFullText(unittest.TestCase):

    def setUp(self):
        self.base = BASE
        self.engine = ENGINE
        self.session = SESSION


        self.entries = []

    def test_fulltext_abuild(self):
        self.assertIsNone(self.base.metadata.create_all(self.engine))

    def test_fulltext_add(self):
        import json
        with open('test_fulltext.json') as fp:
            bulk = json.load(fp)
            for entry in bulk:
                self.entries.append(RecipeReviewModel(
                                entry['commentor'],
                                entry['review']))
                self.entries.append(RecipeReviewModelForMigration(
                                entry['commentor'],
                                entry['review']))

        for entry in self.entries:
            self.assertIsNone( self.session.add(entry))
        self.session.commit()
    def test_fulltext_form_query(self):
        FullTextSearch('spam', RecipeReviewModel)
        FullTextSearch('spam', RecipeReviewModelForMigration)

    def test_fulltext_query(self):
        full = self.session.query(RecipeReviewModel).filter(FullTextSearch('spam', RecipeReviewModel))
        self.assertEqual(full.count(), 2,)
        full = self.session.query(RecipeReviewModelForMigration).filter(FullTextSearch('spam', RecipeReviewModelForMigration))
        self.assertEqual(full.count(), 2,)
        raw = self.session.execute('SELECT * FROM {0} WHERE MATCH (commentor, review) AGAINST ("spam")'.format(RecipeReviewModel.__tablename__))
        self.assertEqual(full.count(), raw.rowcount, 'Query Test Failed')

    def test_fulltext_qutoe_query(self):
        full = self.session.query(RecipeReviewModel).filter(FullTextSearch('"parrot can"', RecipeReviewModel, FullTextMode.BOOLEAN))
        self.assertEqual(full.count(), 2,)
        full = self.session.query(RecipeReviewModelForMigration).filter(FullTextSearch('"parrot can"', RecipeReviewModelForMigration, FullTextMode.BOOLEAN))
        self.assertEqual(full.count(), 2,)
        raw = self.session.execute("""SELECT * FROM {0} WHERE MATCH (commentor, review) AGAINST ('"parrot can"' IN BOOLEAN MODE)""".format(RecipeReviewModel.__tablename__))
        self.assertEqual(full.count(), raw.rowcount)

    @unittest.skipIf(not 'mroonga' in MYSQL_ENGINES, 'mroonga engines not available')
    def test_fulltext_cjk_query(self):
        cjk = self.session.query(RecipeReviewModel).filter(
                                  FullTextSearch('中国人'.decode('utf8'),
                                                 RecipeReviewModel))
        cjk = self.session.query(RecipeReviewModelForMigration).filter(
                                  FullTextSearch('中国人'.decode('utf8'),
                                                 RecipeReviewModelForMigration))


    def test_fulltext_query_natural_mode(self):
        full = self.session.query(RecipeReviewModel).filter(FullTextSearch('spam', RecipeReviewModel, FullTextMode.NATURAL))
        self.assertEqual(full.count(), 2,)
        full = self.session.query(RecipeReviewModelForMigration).filter(FullTextSearch('spam', RecipeReviewModelForMigration, FullTextMode.NATURAL))
        self.assertEqual(full.count(), 2,)
        raw = self.session.execute('SELECT * FROM {0} WHERE MATCH (commentor, review) AGAINST ("spam" IN NATURAL LANGUAGE MODE)'.format(RecipeReviewModel.__tablename__))
        self.assertEqual(full.count(), raw.rowcount, 'Query Test Failed')

    def test_fulltext_query_boolean_mode(self):
        full = self.session.query(RecipeReviewModel).filter(FullTextSearch('spa*', RecipeReviewModel, FullTextMode.BOOLEAN))
        self.assertEqual(full.count(), 3,)
        full = self.session.query(RecipeReviewModelForMigration).filter(FullTextSearch('spa*', RecipeReviewModelForMigration, FullTextMode.BOOLEAN))
        self.assertEqual(full.count(), 3,)
        raw = self.session.execute('SELECT * FROM {0} WHERE MATCH (commentor, review) AGAINST ("spa*" IN BOOLEAN MODE)'.format(RecipeReviewModel.__tablename__))
        self.assertEqual(full.count(), raw.rowcount, 'Query Test Failed')

    def test_github_issue_9(self):
        # https://github.com/mengzhuo/sqlalchemy-fulltext-search/issues/9
        full = self.session.query(RecipeReviewModel).filter(RecipeReviewModel.id >= 1)
        full = full.filter(FullTextSearch('the -rainbow', RecipeReviewModel, FullTextMode.BOOLEAN)).limit(20)
        self.assertEqual(full.count(), 0)
        raw = self.session.execute('SELECT * FROM {0} WHERE MATCH (commentor, review) AGAINST ("the -rainbow" IN BOOLEAN MODE)'.format(RecipeReviewModel.__tablename__))
        self.assertEqual(full.count(), raw.rowcount, 'Query Test Failed')

        full = self.session.query(RecipeReviewModelForMigration).filter(RecipeReviewModelForMigration.id >= 1)
        full = full.filter(FullTextSearch('the -rainbow', RecipeReviewModelForMigration, FullTextMode.BOOLEAN)).limit(20)
        self.assertEqual(full.count(), 0)
        raw = self.session.execute('SELECT * FROM {0} WHERE MATCH (commentor, review) AGAINST ("the -rainbow" IN BOOLEAN MODE)'.format(RecipeReviewModelForMigration.__tablename__))
        self.assertEqual(full.count(), raw.rowcount, 'Query Test Failed')


    def test_fulltext_query_query_expansion_mode(self):
        full = self.session.query(RecipeReviewModel).filter(FullTextSearch('spam', RecipeReviewModel, FullTextMode.QUERY_EXPANSION))
        self.assertEqual(full.count(), 3,)
        full = self.session.query(RecipeReviewModelForMigration).filter(FullTextSearch('spam', RecipeReviewModelForMigration, FullTextMode.QUERY_EXPANSION))
        self.assertEqual(full.count(), 3,)
        raw = self.session.execute('SELECT * FROM {0} WHERE MATCH (commentor, review) AGAINST ("spam" WITH QUERY EXPANSION)'.format(RecipeReviewModel.__tablename__))
        self.assertEqual(full.count(), raw.rowcount, 'Query Test Failed')

if __name__ == '__main__':
    unittest.main()



import traceback

from sekg.graph.accessor import GraphAccessor


class IndexGraphAccessor(GraphAccessor):
    __CYPHER_CREATE_INDEX = "CREATE INDEX ON :`{label}`(`{property_name}`)"
    __CYPHER_DROP_INDEX = "DROP INDEX ON :`{label}`(`{property_name}`)"

    def create_index(self, label, property_name):
        """
        create an index in one specific property with one label
        :param label: the label name to create index with
        :param property_name: the property name to index with
        :return:
        """
        query = self.__CYPHER_CREATE_INDEX.format(label=label, property_name=property_name)

        try:
            result = self.graph.evaluate(query)
            return result
        except Exception:
            traceback.print_exc()
            print("create index fail for %s" % query)
            return None

    def drop_index(self, label, property_name):
        """
        drop an index in one specific property with one label
        :param label: the label name to create index with
        :param property_name: the property name to index with
        :return:
        """
        query = self.__CYPHER_DROP_INDEX.format(label=label, property_name=property_name)

        try:
            result = self.graph.evaluate(query)
            return result
        except Exception:
            traceback.print_exc()
            print("drop index fail for %s" % query)
            return None



import time
from unittest import TestCase

from py2neo import Graph

from sekg.graph.accessor import GraphAccessor
from sekg.graph.creator import NodeBuilder
from sekg.graph.factory import GraphInstanceFactory


class TestGraphAccessor(TestCase):
    test_server_name = "LocalHost"
    test_config_path = "neo4j_config.json"
    test_server_id = 3

    def test_get_graph(self):
        factory = GraphInstanceFactory(self.test_config_path)
        graph = factory.create_py2neo_graph_by_server_id(self.test_server_id)
        accessor = GraphAccessor(graph=graph)
        self.assertTrue(isinstance(accessor.graph, Graph))

    def test_is_connect(self):
        factory = GraphInstanceFactory(self.test_config_path)
        graph = factory.create_py2neo_graph_by_server_id(self.test_server_id)
        accessor = GraphAccessor(graph=None)
        self.assertFalse(accessor.is_connect())
        accessor = GraphAccessor(graph=graph)
        self.assertTrue(accessor.is_connect())

    def test_create_or_update_node(self):
        factory = GraphInstanceFactory(self.test_config_path)
        graph = factory.create_py2neo_graph_by_server_id(self.test_server_id)

        accessor = GraphAccessor(graph=graph)
        self.assertTrue(accessor.is_connect())

        node = NodeBuilder().add_entity_label().add_label("api class").add_one_property(property_name="api id",
                                                                                        property_value=4).add_one_property(
            property_name="api name", property_value="java.lang.Object").build()

        node = accessor.create_or_update_node(node, primary_label="test", primary_property="api id")
        print(node)

        self.assertTrue(accessor.is_remote(node))
        self.assertEqual(node.labels, {"api class", "entity"})
        self.assertEqual(node["api name"], "java.lang.Object")
        self.assertEqual(node["api id"], 4)

        graph.delete(node)

    def test_update_remote_node(self):
        factory = GraphInstanceFactory(self.test_config_path)
        graph = factory.create_py2neo_graph_by_server_id(self.test_server_id)

        accessor = GraphAccessor(graph=graph)
        self.assertTrue(accessor.is_connect())

        node = NodeBuilder().add_entity_label().add_label("api class").add_one_property(property_name="api id",
                                                                                        property_value=4).add_one_property(
            property_name="api name", property_value="java.lang.Object").build()

        node = accessor.create_or_update_node(node, primary_label="test", primary_property="api id")
        print(node)

        self.assertTrue(accessor.is_remote(node))
        self.assertEqual(node.labels, {"api class", "entity"})
        self.assertEqual(node["api name"], "java.lang.Object")
        self.assertEqual(node["api id"], 4)

        node = accessor.update_remote_node(remote_node=node, property_dict={"api id": 5, "version": "1.0"},
                                           labels={"api interface", "api", "entity"})

        self.assertTrue(accessor.is_remote(node))
        self.assertEqual(node.labels, {"api interface", "api", "entity"})
        self.assertEqual(node["api name"], "java.lang.Object")
        self.assertEqual(node["api id"], 5)
        self.assertEqual(node["version"], '1.0')

        graph.delete(node)

    def test_find_node(self):
        factory = GraphInstanceFactory(self.test_config_path)
        graph = factory.create_py2neo_graph_by_server_name(self.test_server_name)
        accessor = GraphAccessor(graph=graph)
        node = NodeBuilder().add_entity_label().add_label("api class").add_one_property(property_name="api id",
                                                                                        property_value=4).add_one_property(
            property_name="api name", property_value="java.lang.Object").build()

        accessor.create_or_update_node(node, primary_label="api class", primary_property="api id")
        node = accessor.find_node(primary_label="api class", primary_property="api id", primary_property_value=4)
        self.assertIsNotNone(node)
        graph.delete(node)
        node = accessor.find_node(primary_label="api class", primary_property="api id", primary_property_value=4)
        self.assertIsNone(node)

    def test_find_relation_by_id(self):
        factory = GraphInstanceFactory(self.test_config_path)
        graph = factory.create_py2neo_graph_by_server_name(self.test_server_name)
        accessor = GraphAccessor(graph=graph)

        start_node = NodeBuilder().add_entity_label().add_label("api class").add_one_property(property_name="api id",
                                                                                              property_value=4).add_one_property(
            property_name="api name", property_value="java.lang.Object").build()

        start_node = accessor.create_or_update_node(start_node, primary_label="api class", primary_property="api id")
        end_node = NodeBuilder().add_entity_label().add_label("api class").add_one_property(property_name="api id",
                                                                                            property_value=6).add_one_property(
            property_name="api name", property_value="java.lang.String").build()
        end_node = accessor.create_or_update_node(end_node, primary_label="api class", primary_property="api id")

        relation = accessor.create_relation_without_duplicate(start_node=start_node, end_node=end_node,
                                                              relation_str="call by")
        self.assertIsNotNone(relation)
        self.assertEqual(accessor.get_relation_type_string(relation), "call by")

        found_relation = accessor.find_relation_by_id(relation.identity)
        self.assertIsNotNone(found_relation)

        self.assertEqual(found_relation.identity, relation.identity)
        old_relation_id = relation.identity
        graph.delete(relation)
        found_relation = accessor.find_relation_by_id(old_relation_id)

        self.assertIsNone(found_relation)

    def test_create_relation_without_duplicate(self):
        factory = GraphInstanceFactory(self.test_config_path)
        graph = factory.create_py2neo_graph_by_server_name(self.test_server_name)
        accessor = GraphAccessor(graph=graph)

        start_node = NodeBuilder().add_entity_label().add_label("api class").add_one_property(property_name="api id",
                                                                                              property_value=4).add_one_property(
            property_name="api name", property_value="java.lang.Object").build()

        start_node = accessor.create_or_update_node(start_node, primary_label="api class", primary_property="api id")
        end_node = NodeBuilder().add_entity_label().add_label("api class").add_one_property(property_name="api id",
                                                                                            property_value=6).add_one_property(
            property_name="api name", property_value="java.lang.String").build()
        end_node = accessor.create_or_update_node(end_node, primary_label="api class", primary_property="api id")

        relation = accessor.create_relation_without_duplicate(start_node=start_node, end_node=end_node,
                                                              relation_str="call by")
        self.assertIsNotNone(relation)
        self.assertEqual(accessor.get_relation_type_string(relation), "call by")
        graph.delete(relation)

    def test_update_metadata(self):
        factory = GraphInstanceFactory(self.test_config_path)
        graph = factory.create_py2neo_graph_by_server_name(self.test_server_name)
        accessor = GraphAccessor(graph=graph)
        node = NodeBuilder().add_entity_label().add_label("api class").add_one_property(property_name="api id",
                                                                                        property_value=4).add_one_property(
            property_name="api name", property_value="java.lang.Object").build()

        node = accessor.create_or_update_node(node, primary_label="api class", primary_property="api id")

        create_time = node["_create_time"]
        update_time = node["_update_time"]
        modify_version = node["_modify_version"]

        self.assertIsNotNone(create_time)
        self.assertIsNotNone(update_time)
        self.assertIsNotNone(modify_version)

        time.sleep(1)

        accessor.update_metadata(node)

        self.assertEqual(modify_version + 1, node["_modify_version"])
        self.assertEqual(create_time, node["_create_time"])
        self.assertNotEqual(update_time, node["_update_time"])

    def test_update_remote_node_by_node(self):
        factory = GraphInstanceFactory(self.test_config_path)
        graph = factory.create_py2neo_graph_by_server_id(self.test_server_id)

        accessor = GraphAccessor(graph=graph)
        self.assertTrue(accessor.is_connect())

        node = NodeBuilder().add_entity_label().add_label("api class").add_one_property(property_name="api id",
                                                                                        property_value=4).add_one_property(
            property_name="api name", property_value="java.lang.Object").build()

        node = accessor.create_or_update_node(node, primary_label="test", primary_property="api id")

        self.assertTrue(accessor.is_remote(node))
        self.assertEqual(node.labels, {"api class", "entity"})
        self.assertEqual(node["api name"], "java.lang.Object")
        self.assertEqual(node["api id"], 4)

        node = accessor.update_remote_node(remote_node=node, property_dict={"api id": 5, "version": "1.0"},
                                           labels={"api interface", "api", "entity"})

        modify_node = NodeBuilder().add_entity_label().add_label("api").add_label("api interface").add_one_property(
            property_name="api id",
            property_value=5).add_one_property(
            property_name="version", property_value="1.0").build()

        accessor.update_remote_node_by_node(remote_node=node, source_node=modify_node)
        self.assertTrue(accessor.is_remote(node))
        self.assertEqual(node.labels, {"api interface", "api", "entity"})
        self.assertEqual(node["api name"], "java.lang.Object")
        self.assertEqual(node["api id"], 5)
        self.assertEqual(node["version"], '1.0')

        graph.delete(node)

    def test_find_node_by_id(self):
        factory = GraphInstanceFactory(self.test_config_path)
        graph = factory.create_py2neo_graph_by_server_name(self.test_server_name)
        accessor = GraphAccessor(graph=graph)
        node = NodeBuilder().add_entity_label().add_label("api class").add_one_property(property_name="api id",
                                                                                        property_value=4).add_one_property(
            property_name="api name", property_value="java.lang.Object").build()

        accessor.create_or_update_node(node, primary_label="api class", primary_property="api id")
        node = accessor.find_node(primary_label="api class", primary_property="api id", primary_property_value=4)
        self.assertIsNotNone(node)

        node = accessor.find_node_by_id(node_id=node.identity)
        self.assertIsNotNone(node)
        graph.delete(node)

    def test_create_node(self):
        factory = GraphInstanceFactory(self.test_config_path)
        graph = factory.create_py2neo_graph_by_server_id(self.test_server_id)

        accessor = GraphAccessor(graph=graph)
        self.assertTrue(accessor.is_connect())

        node = accessor.find_node(primary_label="api class", primary_property="api id", primary_property_value=8)
        self.assertIsNone(node)

        node = NodeBuilder().add_entity_label().add_label("api class").add_one_property(property_name="api id",
                                                                                        property_value=8).add_one_property(
            property_name="api name", property_value="java.lang.String").add_one_property(property_name="type",
                                                                                          property_value="int").build()

        node = accessor.create_node(node)

        self.assertTrue(accessor.is_remote(node))
        self.assertEqual(node.labels, {"api class", "entity"})
        self.assertEqual(node["api name"], "java.lang.String")
        self.assertEqual(node["api id"], 8)
        self.assertEqual(node["type"], "int")

        graph.delete(node)



from sekg.graph.accessor import GraphAccessor


class MetadataGraphAccessor(GraphAccessor):
    """
    query the graph metadata
    """
    # todo: complete this accessor and pass test
    def get_max_id_for_node(self):
        """
            get the max id of Node in the whole graph
            :return: the max id
            """

        query = 'MATCH (n) return max(ID(n))'
        result = self.graph.evaluate(query)
        return result


    def get_max_id_for_relation(self):
        """
        get the max id of relation in the whole graph
        :return: the max id
        """
        query = 'MATCH ()-[n]-() return max(ID(n))'
        result = self.graph.evaluate(query)
        return result

    def get_node_num(self,label):
        pass
        # todo:
    def get_relation_num(self):
        pass
        # todo:
    def get_label_set(self):
        pass
        # todo:
    def get_relation_set(self):
        pass
        # todo:


import json
from abc import ABCMeta, abstractmethod

import gensim
from sekg.graph.accessor import GraphAccessor
from sekg.graph.metadata_accessor import MetadataGraphAccessor

"""
this package is used to export a neo4j instance to Memory to build a copy of graph.
You can used it to train graph vector or to query graph 
"""


class NodeInfo(object):
    """
    a basic abstract class for NodeInfo
    """
    __metaclass__ = ABCMeta
    PRIVATE_PROPERTY = {
        "lastrevid",
        "_create_time",
        "_update_time",
        "_modify_version",
        "modified",
        "logo image"
    }

    def __init__(self, node_id, labels, properties):
        self.labels = labels
        self.properties = properties
        self.node_id = node_id

    @abstractmethod
    def get_main_name(self):
        pass

    @abstractmethod
    def get_all_names(self):
        return []

    def get_all_names_str(self):
        return " , ".join(self.get_all_names())

    @abstractmethod
    def get_all_valid_attributes(self):
        valid_attribute_pairs = []

        for property_name in self.properties.keys():
            if self.is_valid_property(property_name):
                value = self.properties[property_name]
                if not value:
                    continue
                valid_attribute_pairs.append((property_name, value))
        return valid_attribute_pairs

    def get_all_valid_attributes_str(self):
        result = []
        valid_attribute_pairs = self.get_all_valid_attributes()
        for (property_name, value) in valid_attribute_pairs:
            result.append(property_name + " : " + value)

        return " , ".join(result)

    @abstractmethod
    def is_valid_property(self, property_name):
        if property_name in NodeInfo.PRIVATE_PROPERTY:
            return False
        return True

    def __repr__(self):
        return "<NodeInfo id=%d labels=%r properties=%r>" % (self.node_id, self.labels, self.properties)


class NodeInfoFactory(object):
    """
    a basic abstract class for NodeInfo
    """
    __metaclass__ = ABCMeta
    """
    a factory to create NodeInfo, can be extend to create Multi type NodeInfo instance by condition
    """

    def __init__(self):
        pass

    @abstractmethod
    def create_node_info(self, node_info_dict):
        """need to implement"""
        return None


class RelationInfo:

    def __init__(self, start_node_info, relation_name, end_node_info):
        self.relation_name = relation_name
        self.end_node_info = end_node_info
        self.start_node_info = start_node_info

    def get_in_relation_str(self):
        return "{start} {r}".format(
            start=self.start_node_info.get_main_name(),
            r=self.relation_name
        )

    def get_out_relation_str(self):
        return "{r} {end}".format(
            r=self.relation_name,
            end=self.end_node_info.get_main_name()
        )

    def get_full_relation_str(self):
        return "{start} {r} {end}".format(start=self.start_node_info.get_main_name(),
                                          r=self.relation_name,
                                          end=self.end_node_info.get_main_name())


class GraphDataReader:
    """
    this class is a graph Data reader, can get the graph exporter not in the raw json,
    but in more clean format, wrap the result to a specific Object NodeInfo or RelationInfo.
    """

    def __init__(self, graph_data, node_info_factory):
        if isinstance(graph_data, GraphData):
            self.graph_data = graph_data
        else:
            self.graph_data = None

        if isinstance(node_info_factory, NodeInfoFactory):
            self.node_info_factory = node_info_factory
        else:
            self.node_info_factory = None

    def get_all_out_relation_infos(self, node_id):
        relation_list = self.graph_data.get_all_out_relation_dict_list(node_id=node_id)

        return self.create_relation_info_list(relation_list)

    def get_all_in_relation_infos(self, node_id):
        relation_list = self.graph_data.get_all_in_relation_dict_list(node_id=node_id)

        return self.create_relation_info_list(relation_list)

    @staticmethod
    def create_relation_info(start_node_info, relation_type, end_node_info):
        return RelationInfo(start_node_info=start_node_info,
                            relation_name=relation_type,
                            end_node_info=end_node_info)

    def create_from_relation_info_dict(self, relation_info_dict):
        graph_data = self.graph_data
        start_node_info_dict = graph_data.get_node_info_dict(relation_info_dict["startId"])
        start_node_info = self.node_info_factory.create_node_info(start_node_info_dict)

        end_node_info_dict = graph_data.get_node_info_dict(relation_info_dict["endId"])
        end_node_info = self.node_info_factory.create_node_info(end_node_info_dict)

        relation_type = relation_info_dict["relationType"]

        return self.create_relation_info(start_node_info=start_node_info, relation_type=relation_type,
                                         end_node_info=end_node_info,
                                         )

    def create_relation_info_list(self, relation_info_dict_list):
        info_list = []
        for r in relation_info_dict_list:
            info_list.append(self.create_from_relation_info_dict(r))
        return info_list

    def get_node_info(self, node_id):
        node_info_dict = self.graph_data.get_node_info_dict(node_id=node_id)
        return self.node_info_factory.create_node_info(node_info_dict)

    def get_all_node_infos(self):
        result = []
        for node_id in self.graph_data.get_node_ids():
            result.append(self.get_node_info(node_id))
        return result



class GraphData(gensim.utils.SaveLoad):
    """
    the store of a graph exporter for
    """

    def __init__(self):
        # two map for
        self.out_relation_map = {}
        self.in_relation_map = {}
        self.id_to_nodes_map = {}

        self.relation_num = 0
        self.node_num = 0

    def clear(self):
        self.out_relation_map = {}
        self.in_relation_map = {}
        self.id_to_nodes_map = {}

        self.relation_num = 0
        self.node_num = 0

    def set_nodes(self, nodes):
        for n in nodes:
            self.add_node(n)

    def add_node(self, node_json):
        node_id = node_json["id"]
        self.id_to_nodes_map[node_id] = node_json
        self.node_num = self.node_num + 1

    def set_relations(self, relations):
        for t in relations:
            self.add_relation(t)

    def add_relation(self, relation_json):
        startId = relation_json["startId"]
        endId = relation_json["endId"]
        if startId in self.out_relation_map:
            self.out_relation_map[startId].append(relation_json)
        else:
            self.out_relation_map[startId] = [relation_json]

        if endId in self.in_relation_map:
            self.in_relation_map[endId].append(relation_json)
        else:
            self.in_relation_map[endId] = [relation_json]
        self.relation_num = self.relation_num + 1

    def save_as_json(self, path):
        temp = {
            "out_relation_map": self.out_relation_map,
            "in_relation_map": self.in_relation_map,
            "id_to_nodes_map": self.id_to_nodes_map
        }
        json.dump(temp, path)

    def get_node_num(self):
        return self.node_num

    def get_relation_num(self):
        return self.relation_num

    def get_node_ids(self):
        return self.id_to_nodes_map.keys()

    def get_relation_pairs(self):
        # todo:cache the result?
        """
        get the relation list in [(startId,endId)] format
        :return:
        """
        pairs = []
        for r_list in self.out_relation_map.values():
            for r in r_list:
                pairs.append((r["startId"], r["endId"]))

        return pairs

    def get_all_out_relation_dict_list(self, node_id):
        if node_id not in self.out_relation_map:
            return []

        return self.out_relation_map[node_id]

    def get_all_in_relation_dict_list(self, node_id):
        if node_id not in self.in_relation_map:
            return []

        return self.in_relation_map[node_id]

    def get_node_info_dict(self, node_id):
        """
        get the node info dict,
        :param node_id: the node id
        :return:
        """
        if node_id not in self.id_to_nodes_map:
            return None
        return self.id_to_nodes_map[node_id]


class DataExporterAccessor(GraphAccessor):
    def get_all_nodes_not_batch(self, node_label):
        try:
            query = 'Match (n:`{node_label}`) return n'.format(node_label=node_label)

            cursor = self.graph.run(query)
            nodes = []
            for record in cursor:
                nodes.append(record["n"])
            return nodes
        except Exception:
            return []

    def get_all_nodes(self, node_label, step=100000):
        metadata_accessor = MetadataGraphAccessor(self)
        max_node_id = metadata_accessor.get_max_id_for_node()

        nodes = []

        for start_id in range(0, max_node_id, step):
            end_id = min(max_node_id, start_id + step)
            nodes.extend(self.get_all_nodes_in_scope(node_label, start_id=start_id, end_id=end_id))
            print("get nodes step %d-%d" % (start_id, end_id))

        return nodes

    def get_all_nodes_in_scope(self, node_label, start_id, end_id):
        try:
            query = 'Match (n:`{node_label}`) where ID(n)>{start_id} and ID(n)<={end_id} return n'.format(
                node_label=node_label, start_id=start_id, end_id=end_id)

            cursor = self.graph.run(query)
            nodes = []
            for record in cursor:
                nodes.append(record["n"])

            return nodes
        except Exception:
            return []

    def get_all_relation(self, node_label, step=200000):
        metadata_accessor = MetadataGraphAccessor(self)
        max_relation_id = metadata_accessor.get_max_id_for_relation()

        relations = []

        for start_id in range(0, max_relation_id, step):
            end_id = min(max_relation_id, start_id + step)
            relations.extend(self.get_all_relation_in_scope(node_label, start_id=start_id, end_id=end_id))
            print("get relation step %d-%d" % (start_id, end_id))
        return relations

    def get_all_relation_in_scope(self, node_label, start_id, end_id):
        try:
            query = 'Match (n:`{node_label}`)-[r]->(m:`{node_label}`) where ID(r)>{start_id} and ID(r)<={end_id} return ID(n) as startId,ID(m) as endId, type(r) as relationType'.format(
                node_label=node_label, start_id=start_id, end_id=end_id)

            cursor = self.graph.run(query)
            data = cursor.data()
            return data
        except Exception:
            return []

    def get_all_relation_not_batch(self, node_label):
        try:
            query = 'Match (n:`{node_label}`)-[r]->(m:`{node_label}`) return ID(n) as startId,ID(m) as endId, type(r) as relationType'.format(
                node_label=node_label)

            cursor = self.graph.run(query)
            data = cursor.data()
            return data
        except Exception:
            return []


class GraphDataExporter:
    """
    export specific data
    """

    def __init__(self):
        pass

    def export_all_graph_data(self, graph, node_label):
        accessor = DataExporterAccessor(graph=graph)
        nodes = accessor.get_all_nodes(node_label=node_label)
        graph_data = GraphData()

        for node in nodes:
            labels = [label for label in node.labels]
            team = {"id": node.identity, "properties": dict(node), "labels": labels}
            graph_data.add_node(team)

        print("load entity complete, num=%d" % len(nodes))
        relations = accessor.get_all_relation(node_label=node_label)
        print("load relation complete,num=%d" % len(relations))
        graph_data.set_relations(relations=relations)

        return graph_data



from unittest import TestCase

from py2neo import Graph, Node

from sekg.graph.factory import GraphInstanceFactory


class TestGraphInstanceFactory(TestCase):

    def test_create_py2neo_graph_by_server_name(self):
        factory = GraphInstanceFactory("neo4j_config.json")
        graph2 = factory.create_py2neo_graph_by_server_id(2)
        self.assertIsNotNone(graph2)
        self.assertTrue(type(graph2) == Graph)

        graph3 = factory.create_py2neo_graph_by_server_id(3)
        self.assertIsNone(graph3)

        graphDysD3 = factory.create_py2neo_graph_by_server_name("DysD3")
        self.assertIsNotNone(graphDysD3)
        self.assertTrue(type(graphDysD3) == Graph)

        graphDysD3 = factory.create_py2neo_graph_by_server_name("DysD3Wrong")
        self.assertIsNone(graphDysD3)

    def test___init__(self):
        factory = GraphInstanceFactory("neo4j_neo4j_config.json")
        configs = [
            {'server_name': 'LocalHostServer', 'server_id': 1, 'host': 'localhost', 'user': 'neo4j',
             'password': '123456', 'http_port': 7474, 'https_port': 7473, 'bolt_port': 7687},
            {'server_name': 'DysD3', 'server_id': 2, 'host': '10.141.221.87', 'user': 'neo4j',
             'password': '123456', 'http_port': 7474, 'https_port': 7473, 'bolt_port': 7687}
        ]
        self.assertEqual("neo4j_config.json",
                         factory.get_config_file_path())
        self.assertEqual(configs,
                         factory.get_configs())

    def test_get_node_id(self):
        factory = GraphInstanceFactory("neo4j_config.json")
        graphDysD3 = factory.create_py2neo_graph_by_server_name("DysD3")

        node = graphDysD3.evaluate("Match (n:entity) where id(n)=4 return n")
        self.assertTrue(type(node) == Node)
        print(node)
        self.assertEqual(node.identity, 4)

        new_node = Node("person", a=1, name="harry")
        print(new_node)
        print(new_node.identity)



import json
import os

from py2neo import Graph


class GraphInstanceFactory:

    def __init__(self, config_file_path):
        """
        init the graph factory by a config path.
        the config json file format example:
        [
            {
                "server_name": "LocalHostServer",
                "server_id": 1,
                "host": "localhost",
                "user": "neo4j",
                "password": "123456",
                "http_port": 7474,
                "https_port": 7473,
                "bolt_port": 7687
            },
            ...
        ]
        :param config_file_path: the config file path
        """
        if not os.path.exists(config_file_path):
            raise IOError("Neo4j config file not exist")
        if not os.path.isfile(config_file_path):
            raise IOError("Neo4j config path is not file")
        if not config_file_path.endswith(".json"):
            raise IOError("Neo4j config file is not json")

        self.config_file_path = config_file_path
        with open(self.config_file_path, 'r') as f:
            self.configs = json.load(f)
        ## todo add more json format check,raise exception when same name or same id config

    def create_py2neo_graph_by_server_name(self, server_name):
        """
        :param server_name: the server name in config file, can be used to find a unique neo4j graph instance location
        :return: the Graph object in py2neo, None if create fail
        """

        for config in self.configs:
            if config["server_name"] == server_name:
                return self.__create_py2neo_graph_by_config(config)
        return None

    def create_py2neo_graph_by_server_id(self, server_id):
        """
        :param server_id: the server id in config file, can be used to find a unique neo4j graph instance location
        :return: the Graph object in py2neo, None if create fail
        """
        for config in self.configs:
            if config["server_id"] == server_id:
                return self.__create_py2neo_graph_by_config(config)
        return None

    def get_configs(self):
        """
        get the config server list
        :return: a list of config
        """
        return self.configs

    def get_config_file_path(self):
        """
        get the config file path
        :return: a string for config file path
        """
        return self.config_file_path

    def __create_py2neo_graph_by_config(self, config):
        return Graph(host=config['host'],
                     port=config['bolt_port'],
                     scheme="bolt",
                     user=config['user'],
                     password=config['password'])
